@@ -34,7 +34,8 @@ enum valid_family {
   truncated_nbinom1_family =502,
   truncated_nbinom2_family =503,
   t_family =600,
-  tweedie_family = 700
+  tweedie_family = 700,
+  lognormal_family = 800
 };
 
 // capitalize Family so this doesn't get picked up by the 'enum' scraper
@@ -66,7 +67,8 @@ enum valid_covStruct {
   gau_covstruct = 6,
   mat_covstruct = 7,
   toep_covstruct = 8,
-  rr_covstruct = 9
+  rr_covstruct = 9,
+  homdiag_covstruct = 10
 };
 
 enum valid_ziPredictCode {
@@ -146,7 +148,7 @@ Type log_inverse_linkfun(Type eta, int link) {
   return ans;
 }
 
-/* log transformed inverse_linkfun without losing too much accuracy */
+/* log transformed (1-inverse_linkfun) without losing too much accuracy */
 template<class Type>
 Type log1m_inverse_linkfun(Type eta, int link) {
   Type ans;
@@ -167,18 +169,18 @@ Type log1m_inverse_linkfun(Type eta, int link) {
 template<class Type>
 Type calc_log_nzprob(Type mu, Type phi, Type eta, Type etad, int family,
 		     int link) {
-  Type ans, s1, s2, s3;
+  Type ans, s1, s2;
   switch (family) {
   case truncated_nbinom1_family:
-    s3 = logspace_add( Type(0), etad);      // log(1. + phi(i)
-    ans = logspace_sub( Type(0), -mu / phi * s3 ); // 1-prob(0)
+    s2 = logspace_add( Type(0), etad);      // log(1. + phi(i)
+    ans = logspace_sub( Type(0), -mu / phi * s2 ); // 1-prob(0)
     break;
   case truncated_nbinom2_family:
     // s1 is repeated computation from main loop ...
     s1 = log_inverse_linkfun(eta, link);          // log(mu)
-    // s3 := log( 1. + mu(i) / phi(i) )
-    s3 = logspace_add( Type(0), s1 - etad );
-    ans = logspace_sub( Type(0), -phi * s3 );
+    // s2 := log( 1. + mu(i) / phi(i) )
+    s2 = logspace_add( Type(0), s1 - etad );
+    ans = logspace_sub( Type(0), -phi * s2 );
     break;
   case truncated_poisson_family:
     ans = logspace_sub(Type(0), -mu);  // log(1-exp(-mu(i))) = P(x>0)
@@ -241,10 +243,12 @@ struct terms_t : vector<per_term_info<Type> > {
   }
 };
 
+// compute log-likelihood of b (conditional modes) conditional on theta (var/cov)
+//  for a specified random-effects term 
 template <class Type>
 Type termwise_nll(array<Type> &U, vector<Type> theta, per_term_info<Type>& term, bool do_simulate = false) {
   Type ans = 0;
-  if (term.blockCode == diag_covstruct){
+  if (term.blockCode == diag_covstruct) {
     // case: diag_covstruct
     vector<Type> sd = exp(theta);
     for(int i = 0; i < term.blockReps; i++){
@@ -255,6 +259,26 @@ Type termwise_nll(array<Type> &U, vector<Type> theta, per_term_info<Type>& term,
     }
     term.sd = sd; // For report
   }
+  else if (term.blockCode == homdiag_covstruct) {
+    // case: homdiag_covstruct
+    Type sd = exp(theta(0));
+    for(int i = 0; i < term.blockReps; i++){
+          for (int j = 0; j < U.rows(); j++) {
+	    ans -= dnorm(Type(U(j,i)), Type(0), sd, true);
+	    if (do_simulate) {
+	      U(j,i) = rnorm(Type(0), sd);
+	    }	      
+	  }
+    }
+    int n = term.blockSize;
+    vector<Type> sdvec(n);
+    for(int i = 0; i < n; i++) {
+      sdvec(i) = sd;
+    }
+    
+    term.sd = sdvec; // For report
+  }
+
   else if (term.blockCode == us_covstruct){
     // case: us_covstruct
     int n = term.blockSize;
@@ -433,6 +457,8 @@ Type termwise_nll(array<Type> &U, vector<Type> theta, per_term_info<Type>& term,
   }
   else if (term.blockCode == rr_covstruct){
     // case: reduced rank
+
+    // computing log-likelihood based on *spherical* (iid N(0,1)) random effects
     for(int i = 0; i < term.blockReps; i++){
       ans -= dnorm(vector<Type>(U.col(i)), Type(0), 1, true).sum();
       if (do_simulate) {
@@ -440,6 +466,10 @@ Type termwise_nll(array<Type> &U, vector<Type> theta, per_term_info<Type>& term,
       }
     }
 
+    // now construct the factor matrix and convert the spherical random
+    //  effects back to the 'data scale', and *replace them* in the U matrix
+
+    // constructing the factor loadings matrix
     int p = term.blockSize;
     int nt = theta.size();
     int rank = (2*p + 1 -  (int)sqrt(pow(2.0*p + 1, 2) - 8*nt) ) / 2 ;
@@ -457,11 +487,14 @@ Type termwise_nll(array<Type> &U, vector<Type> theta, per_term_info<Type>& term,
       }
     }
 
+    // transforming u to b by multiplying by the loadings matrix
     for(int i = 0; i < term.blockReps; i++){
       vector<Type> usub = U.col(i).segment(0, rank);
       U.col(i) = Lambda * usub;
     }
 
+    // computing the correlation matrix and std devs
+    // (the same D^(-1/2) L L^T D^(-1/2) transformation that we use for correlations
     term.fact_load = Lambda;
     if(isDouble<Type>::value) {
       term.corr = Lambda * Lambda.transpose();
@@ -612,6 +645,8 @@ Type objective_function<Type>::operator() ()
 	// n.b. sigma() calculation is special-cased for Gaussian
 	// (exp(0.5*pl$betad), all other families except Gamma
 	//  use exp(pl$betad)
+	// so phi = variance, not SD
+	// (FIXME: ?? why ??)
         tmp_loglik = dnorm(yobs(i), mu(i), sqrt(phi(i)), true);
         SIMULATE{yobs(i) = rnorm(mu(i), sqrt(phi(i)));}
         break;
@@ -654,6 +689,21 @@ Type objective_function<Type>::operator() ()
 
 	  // std::cout << "middle " << asDouble(eta(i)) << " " << asDouble(psi(0)) << " " << asDouble(psi(1)) << " " << asDouble(s3) << " " << asDouble(tmp_loglik) << " " << asDouble(s1) << " " << asDouble(s2) << " " << asDouble(mu(i)) << " " << asDouble(phi(i)) << std::endl;
 	}
+	SIMULATE{
+	  s3 = invlogit(psi(0) - eta(i));
+	  if (runif(Type(0), Type(1)) < s3) {
+	    yobs(i) = 0;
+	  } else {
+	    s3 = invlogit(eta(i) - psi(1));
+	    if (runif(Type(0), Type(1)) < s3) {
+	      yobs(i) = 1;
+	    } else {
+	      s1 = mu(i)*phi(i);
+	      s2 = (Type(1)-mu(i))*phi(i);
+	      yobs(i) = rbeta(s1, s2);
+	    }
+	  }
+	}
 	break;
       case betabinomial_family:
         // Transform to logit scale independent of link
@@ -751,6 +801,21 @@ Type objective_function<Type>::operator() ()
         SIMULATE {
           yobs(i) = glmmtmb::rtweedie(s1, s2, s3);
         }
+	break;
+      case lognormal_family:
+	// parameterized in terms of mean and SD on *data* scale, i.e.
+	// mu = exp(logmu + logsd^2/2)
+	// sd = sqrt((exp(logsd^2)-1)*exp(2*logmu + logsd^2)) = mu*sqrt(exp(logsd^2)-1)
+	// 1+(sd/mu)^2 = exp(logsd^2)
+	// logsd = sqrt(log(1+(sd/mu)^2))
+	// logmu = log(mu)- 
+        s1 = log1p(pow(phi(i)/mu(i), 2.0)); //log1p(x) = log(1 + x), log-scale var
+        s2 = log(mu(i)) - s1/2; //log-scale mean
+        // s2 = log(mu(i)*mu(i)) - log(mu(i)*mu(i) + phi(i)*phi(i))/Type(2.0); //from Wikipedia
+        s3 = sqrt(s1); //log-scale sd
+
+	tmp_loglik = dnorm(log(yobs(i)), s2, s3, true) - log(yobs(i));
+	// FIXME: simulate method?
         break;
       case t_family:
         s1 = (yobs(i) - mu(i))/phi(i);
@@ -779,6 +844,7 @@ Type objective_function<Type>::operator() ()
         SIMULATE{yobs(i) = yobs(i)*rbinom(Type(1), Type(1)-pz(i));}
       }
       tmp_loglik *= weights(i);
+
       // Add up
       jnll -= keep(i) * tmp_loglik;
     }
